/* DataList - template parameter should be like x0, x1, x2, x3...
 *  compare v against each x, to check if list contains or not contains v
 *
 * Usage:
 *  using DL = DataList<3, 6, 8, 19>;
 *  auto contains = DL::Contains(3); // contains = true
 *  auto contains = DL::Contains(4); // contains = flase
 */
constexpr size_t MAX_DATA_LIST = 32;
template<auto... vs>
struct DataList {
    static_assert(sizeof...(vs) <= MAX_DATA_LIST, "DataList too much, use std::set instead");
    template<typename T>
    constexpr static bool Contains(T val)
    {
        return ((val == vs) || ...);
    }

    template<typename T>
    constexpr static bool NotContains(T val)
    {
        return !Contains(val);
    }
};

/* DataPairs - template parameter should be like x0, y0, x1, y1, x2, y2, ..., z
 *  if v matches one of x, return corresponding y, else return z as default
 *
 * Usage:
 *  using DP = DataPairs<1, 10, 2, 20, 0>;
 *  auto v = DP::ValueOf(1); // v == 10
 *  auto w = DP::ValueOf(3); // w == 0
 */
constexpr size_t MAX_DATA_PAIRS = 16;
template<auto x, auto y, auto z, auto... vs>
struct DataPairs {
    static_assert(sizeof...(vs) %2 == 0, "DataPairs must be odd, and >= 3");
    static_assert(sizeof...(vs) < MAX_DATA_PAIRS * 2, "DataPairs too much, use std::map instead");

    template<typename T>
    constexpr static auto ValueOf(T val)
    {
        if constexpr (sizeof...(vs) == 0) {
            return val == x ? y : z;
        } else {
            return val == x ? y : DataPairs<z, vs...>::ValueOf(val);
        }
    }
};
