#include <atomic>
#include <mutex>
#include <functional>
#include <memory>
#include <vector>
#include <queue>
#include <thread>
#include <condition_variable>

namespace Add {
class Threadpool
{
  class State {
  public:
    State(Threadpool* pool): pool_(pool) {}
    virtual bool Init() { return false; }
    virtual bool Start() { return false; }
    virtual bool Stop() { return false; }
    void Done() { bool expected = true; pool_->stateProcessing_.compare_exchange_strong(expected, false); }
  protected:
    Threadpool* pool_ { nullptr };
  };
  template<typename T> static State* GetStateInstance(Threadpool* pool) {
    static T state(pool);
    return &state;
  }
  class DefaultState final : public State {
  public:
    using State::State;
    bool Init() override;
  };
  class InitedState final : public State {
  public:
    using State::State;
    bool Start() override;
  };
  class StartedState final : public State {
  public:
    using State::State;
    bool Stop() override;
  };

  friend State;
  friend DefaultState;
  friend InitedState;
  friend StartedState;
public:
  /*
   * 构造线程池对象
   */
  Threadpool() { state_ = GetStateInstance<DefaultState>(this); }
  /*
   * 销毁线程池对象
   */
  ~Threadpool() {
    if (exit_) {
        return;
    }
    Stop(); // not care the return value
  }

  /*
   * 初始化线程池
   * 参数:
   *   number:  int, 线程池容纳的线程数目
   * 返回值:
   *   true: 成功初始化
   *   false: 初始化失败。可能已经初始化过了。
   */
  bool Init(size_t number) {
    if (!state_.load()->Init()) {
      return false;
    }
    threadsCnt_ = number;
    threads_.resize(number);
    state_.load()->Done();
    return true;
  }
 
  /*
   * 启动线程池里各个线程
   * 启动后，各个线程会阻塞，等待任务到来。处理完任务，又会阻塞，等待其他任务
   * 参数:
   *   无
   * 返回值:
   *   true: 成功启动
   *   false: 启动失败。可能没有初始化，或者已经启动过了
   */
  bool Start() {
    if (!state_.load()->Start()) {
      return false;
    }
    std::unique_lock<std::mutex> lock(taskMutex_);
    exit_ = false;
    lock.unlock();
    auto num = threadsCnt_.load();
    for (size_t i = 0; i < num; i++) {
      threads_[i] = std::thread(&Threadpool::Run, this);
    }
    state_.load()->Done();
    return true;
  }
  /*
   * 向线程池递交任务
   * 仅仅递交任务马上返回，不会等待任务完成
   * 参数:
   *   无
   * 返回值:
   *   true: 成功递交任务
   *   false: 递交任务失败，可能线程在某处已经调用Stop了。
   */
  bool PostTask(std::function<void()> task) {
    bool result = false;
    {
      std::unique_lock<std::mutex> lock(taskMutex_);
      if (!exit_) {
        tasks_.push(task);
	taskCnt_++;
        result = true;
      }
    }
    if (result) {
      taskCond_.notify_one();
    }
    return result;
  }
  /*
   * 等待各个任务处理结束
   * 某些使用场景，在递交任务后，可能需要各个任务完成
   * 参数:
   *   无
   * 返回值:
   *   无
   */
  void WaitTasks() {
    std::unique_lock<std::mutex> lock(taskMutex_);
    finishCond_.wait(lock, [this] { return exit_ || taskCnt_ == 0; });
  }
  /*
   * 取消已经递交的任务
   * 任务递交后，如果尚未执行，可以用此接口取消。如果已经执行，则不会。
   * 参数:
   *   无
   * 返回值:
   *   无
   */
  void CancelTasks() {
    std::unique_lock<std::mutex> lock(taskMutex_);
    size_t leaveCnt = tasks_.size();
    taskCnt_ -= leaveCnt;
    // clear tasks
    std::queue<std::function<void()>> temp;
    tasks_.swap(temp);
  }
  /*
   * 停止线程池里各个线程
   * 此接口会阻塞，直到各个线程合并。如果此后需要用线程池，需要重新Start
   * 参数:
   *   无
   * 返回值:
   *   true: 成功停止
   *   false: 停止失败。可能没有初始化，或者已经停止过了
   */
  bool Stop() {
    if (!state_.load()->Stop()) {
      return false;
    }
    std::unique_lock<std::mutex> lock(taskMutex_);
    exit_ = true;
    lock.unlock();
    taskCond_.notify_all();
    for (auto it = threads_.begin(); it != threads_.end(); ++it) {
      it->join();
    }
    state_.load()->Done();
    return true;
  }
  /*
   * 获取线程池线程数目
   * 参数:
   *   无
   * 返回值:
   *   size_t的数值，代表线程池初始化时，传过来的数目，即其中的线程数
   */
  size_t GetThreadsNumber() const {
    return threadsCnt_.load();
  }
  /*
   * 获取线程池任务数目
   * 参数:
   *   无
   * 返回值:
   *   size_t的数值，代表递交到线程池，尚未来得及处理完的数目
   */
  size_t GetTasksNumber() const {
    return taskCnt_.load();
  }

private:
  Threadpool(const Threadpool&) = delete;
  Threadpool& operator=(const Threadpool&) = delete;
  void Run() {
    while (!exit_) {
      std::function<void()> task;
      {
        std::unique_lock<std::mutex> lock(taskMutex_);
        taskCond_.wait(lock, [this] { return exit_ || !this->tasks_.empty(); });
	if (exit_) {
	  break;
	}
        task = tasks_.front();
        tasks_.pop();
      }
      task();
      std::unique_lock<std::mutex> lock(taskMutex_);
      taskCnt_--;
      lock.unlock();
      if (taskCnt_ == 0) {
        finishCond_.notify_all();
      }
    }
  }

private:
  std::atomic<State*> state_ { nullptr };
  std::vector<std::thread> threads_;
  std::queue<std::function<void()>> tasks_;
  std::mutex taskMutex_;
  std::condition_variable taskCond_;
  std::condition_variable finishCond_;
  std::atomic<size_t> threadsCnt_ { 0 };
  std::atomic<size_t> taskCnt_ { 0 };
  std::atomic_bool exit_ { true };
  std::atomic_bool stateProcessing_ { false };
};

inline bool Threadpool::DefaultState::Init() {
  bool expected = false;
  if (!pool_->stateProcessing_.compare_exchange_strong(expected, true)) {
    return false;
  }
  pool_->state_ = Threadpool::GetStateInstance<InitedState>(pool_);
  return true;
}
inline bool Threadpool::InitedState::Start() {
  bool expected = false;
  if (!pool_->stateProcessing_.compare_exchange_strong(expected, true)) {
    return false;
  }
  pool_->state_ = Threadpool::GetStateInstance<StartedState>(pool_);
  return true;
}
inline bool Threadpool::StartedState::Stop() {
  bool expected = false;
  if (!pool_->stateProcessing_.compare_exchange_strong(expected, true)) {
    return false;
  }
  pool_->state_ = Threadpool::GetStateInstance<InitedState>(pool_);
  return true;
}
} // namespace Add

